Class: Otto::ResponseHandlers::AutoHandler
- Inherits:
-
BaseHandler
- Object
- BaseHandler
- Otto::ResponseHandlers::AutoHandler
- Defined in:
- lib/otto/response_handlers.rb
Overview
Auto-detection handler that chooses appropriate handler based on context
Class Method Summary collapse
Class Method Details
.handle(result, response, context = {}) ⇒ Object
85 86 87 88 89 |
# File 'lib/otto/response_handlers.rb', line 85 def self.handle(result, response, context = {}) # Auto-detect based on result type and request context handler_class = detect_handler_type(result, response, context) handler_class.handle(result, response, context) end |